/* File: xbm2grob.c
** Author: John C. Orthoefer (Internet: jco@ufl.edu)
** Date: 3 Aug 1990
** Program: Converts Xbitmaps to GROB (Graphics Object) format for 
**		the HP 48sx
** History: 
**	6 Aug 1990 - Cleaned up code and released
**	3 Aug 1990 - Code version 1.0
*/
#include <stdio.h>
#include <strings.h>

#define MaxNameSize 256
#define PARAMS "v"

/* This is a table of backward nibbles */
int table[] = { 0x0, 0x8, 0x4, 0xc, 
		0x2, 0xa, 0x6, 0xe, 
		0x1, 0x9, 0x5, 0xd,
		 0x3, 0xb, 0x7, 0xf };

int verbose = 0;

main(argc, argv)
int argc;
char *argv[];
{
	FILE *xbitmap;
	FILE *outfile;
	int i;
	extern int optind;
	char filename[MaxNameSize];
	char c;

	while ((c = getopt(argc, argv, PARAMS)) != EOF) {
		switch(c) {
		default:
		case '?': 
			fprintf(stderr, "Usage: %s [-v] file [file ...]\n", argv[0]);
			exit(1);
			break;
		case 'v': verbose = 1;
			break;
		}
	}

	for (i = optind; i < argc; i++) {
		if ((xbitmap = fopen(argv[i], "r")) == NULL) {
			perror(argv[0]);
			exit(1);
		} else {
		strcpy(filename, argv[i]);
		strcat(filename, ".grob");
		if ((outfile = fopen(filename, "w")) == NULL) {
			perror(argv[0]);
			exit(1);
		} else {
			if (verbose)
				printf("XBM: %s\nGROB: %s\n", argv[i], filename);
			convert( xbitmap, outfile);
			close(xbitmap);
			close(outfile);
		}
		}
	}
}

convert( xbitmap, outfile)
FILE *xbitmap;
FILE *outfile;
{
	int width, height;
	int i;
	int data;

/* scan for #define filename_width */
	while (1 != fscanf(xbitmap, "_width %d\n", &width)) {
		fgetc(xbitmap);
	}

/* scan for #define filename_height */
	while (1 != fscanf(xbitmap, "_height %d\n", &height)) {
		fgetc(xbitmap);
	}

/* verbose option */
	if (verbose)
		printf("X-bitmap: Height = %d, Width = %d\n", height, width);

	fprintf(outfile, "GROB %d %d ", width, height);

	while('{' != fgetc(xbitmap)) {
	}

	i = 0;
	while (fgetc(xbitmap) != '}') {
		if (1 == fscanf(xbitmap, " 0x%x", &data)) { 
			i++;
			fprintf(outfile, "%1X%1X", data & 0xf, (data & 0xf0) >> 4);
#ifdef DEBUG
			printf("%x(%1X%1X)  ", data, data & 0xf, (data & 0xf0) >> 4);
#endif
		}
	}

	if (verbose) 
		printf("converted: %d\n", i);
	fclose(xbitmap);
	fclose(outfile);
}
